@@ -23,10 +23,16 @@ case "$TRAVIS_OS_NAME" in
     sudo apt-get -q install --no-install-recommends -y curl git gnupg-agent locales pinentry-curses pkg-config rsync software-properties-common
     sudo apt-get -q clean
     sudo rm -rf /var/lib/apt/lists/*
-    curl -L https://www-us.apache.org/dist/yetus/0.8.0/yetus-0.8.0-bin.tar.gz | tar xvz -C /tmp/
+
+    # Only Yetus 0.9.0+ supports `ADD` and `COPY` commands in Dockerfile
+    curl -L https://www-us.apache.org/dist/yetus/0.10.0/apache-yetus-0.10.0-bin.tar.gz | tar xvz -C /tmp/
+    # A dirty workaround to disable the Yetus robot for TravisCI,
+    # since it'll cancel the changes that .travis/script.sh will do,
+    # even if the `--dirty-workspace` option is specified.
+    rm /tmp/apache-yetus-0.10.0/lib/precommit/robots.d/travisci.sh
     ;;
 "windows")
-    choco install dotnetcore-sdk --version 2.2.300
+    choco install dotnetcore-sdk --version 3.1.101
     ;;
 *)
     echo "Invalid PLATFORM"
