@@ -19,8 +19,10 @@ set -e
 
 case "$TRAVIS_OS_NAME" in
 "linux")
-    sed -i.bak "s/openjdk:8/openjdk:${JAVA}/" share/docker/Dockerfile
-    /tmp/yetus-0.8.0/bin/test-patch --plugins=buildtest --java-home=/usr/local/openjdk-"${JAVA}" --user-plugins=share/precommit/ --run-tests --empty-patch --docker --dockerfile=share/docker/Dockerfile --dirty-workspace --verbose=true
+    # Workaround for Yetus. For now, Yetus assumes the directory in which Dockerfile is placed is the docker context.
+    # So the Dockerfile should be here to refer to other subdirectories than share/docker from inside the Dockerfile.
+    cp share/docker/Dockerfile .
+    /tmp/apache-yetus-0.10.0/bin/test-patch --plugins=buildtest --java-home=/usr/local/openjdk-"${JAVA}" --user-plugins=share/precommit/ --run-tests --empty-patch --docker --dockerfile=Dockerfile --dirty-workspace
     ;;
 "windows")
     ./lang/csharp/build.sh test
